/* 
 * File:   main.c
 * Author:  Ing. Peter Adamec
 * Company: MCU Slovel s.r.o.
 * Contact: mcu@mcu.sk
 *
 * Created on Sunday, 2014, february 2, 0:46
 */

#include <stdio.h>
#include <stdlib.h>
#include <pic16f1508.h>
#include <xc.h>
#include "main.h"
#include "lcd_pic.h"

#pragma config FOSC=INTOSC, WDTE=OFF, MCLRE=ON, BOREN=OFF, BORV=LO, LVP=OFF


volatile unsigned char meas_timing;
volatile unsigned char offset_timing;
volatile unsigned char view_flag;
volatile unsigned int vin_input;
volatile unsigned int vout_input;
volatile unsigned int iout_input;
volatile unsigned int vin_zero;
volatile unsigned int vout_zero;
volatile unsigned int iout_zero;
volatile unsigned int fvr_input;


void adc_zero(void) {
    
    TRISAbits.TRISA2=0;
    TRISCbits.TRISC0=0;
    TRISCbits.TRISC1=0;
    LATAbits.LATA2=0;
    LATCbits.LATC0=0;
    LATCbits.LATC1=0;
    }

void adc_open(void) {
    TRISAbits.TRISA2=1;
    TRISCbits.TRISC0=1;
    TRISCbits.TRISC1=1;
    LATAbits.LATA2=0;
    LATCbits.LATC0=0;
    LATCbits.LATC1=1;
    
    }

void adc_init(void) {
    LATAbits.LATA2=0;
    LATCbits.LATC0=0;
    LATCbits.LATC1=0;
    TRISAbits.TRISA2=1;
    TRISCbits.TRISC0=1;
    TRISCbits.TRISC1=1;
    ANSELAbits.ANSA2=1;
    ANSELCbits.ANSC0=1;
    ANSELCbits.ANSC1=1;

    FVRCON = 0b11000001;    // fixed voltage enable, capsense reference 2.048V, adc reference 2.048V    -   FIXED VOLTAGE REFERENCE
    ADCON2=0b00000000;	// no auto conversion
    ADCON1=0b10010000;      // ADFM=1 FOSC/8 Vrpos connected to Vdd
    ADCON0bits.ADON=1;
}


unsigned int read_adc(unsigned char input,unsigned char oversampling)	{
    unsigned int adc_val;
    unsigned char i;
ADCON0=input;
ADCON0bits.ADON=1;
ADCON0bits.ADGO=1;
while (ADCON0 & 0b10) ;		// wait for complete (DONE bit active low)
adc_val=0;
for (i=0;i<oversampling;i++)
    {
    ADCON0bits.ADGO=1;
    while (ADCON0 & 0b10) ;		// wait for complete (DONE bit active low)
    adc_val += ADRES;
    }
return adc_val;

}

void tmr1_init(void)	{
T1CONbits.TMR1CS0=0;
T1CONbits.TMR1CS1=0;
T1CONbits.T1CKPS0=1;
T1CONbits.T1CKPS1=1;
T1CONbits.TMR1ON=1;
TMR1=15535;         // overflow 10HZ
PIE1bits.TMR1IE=1;  //timer 1 interupt enable
INTCONbits.PEIE=1;  // peripheral interupt enable
INTCONbits.GIE=1;   // global interupt enable
}

void interrupt ISR (void)   // interupt je nestandardne dlhy ale v tejto aplikacii to nevadi
{
unsigned char i;

if (PIR1bits.TMR1IF)
    {
    meas_timing &= 0b1111;

    if (meas_timing==14) {
            vout_input = read_adc(vout_ch,32);
            vout_input = (vout_input-vout_zero)>>5;
            iout_input = read_adc(iout_ch,32);     // prvy prevod po zmene kanalu moze byt nahodne cislo
            iout_input = (iout_input-iout_zero)>>5;
            vin_input = read_adc(vin_ch,32);     // prvy prevod po zmene kanalu moze byt nahodne cislo
            vin_input = (vin_input-vin_zero)>>5;
            view_flag = 1;
            adc_zero(); // zacne vybijat 100nF kondenzator pre meranie offsetu za 50ms
            }
    else if(meas_timing==15) {
            vout_zero = read_adc(vout_ch,32);
            iout_zero = read_adc(iout_ch,32);
            vin_zero  = read_adc(vin_ch,32);
            fvr_input = read_adc(fvr_ch,32);
            fvr_input >>= 5;
            adc_open();
            }
    else if ( (meas_timing & 0x01) == 0 )
            {
            vout_input = read_adc(vout_ch,32);
            vout_input = (vout_input-vout_zero)>>5;
            iout_input = read_adc(iout_ch,32);     
            iout_input = (iout_input-iout_zero)>>5;
            vin_input = read_adc(vin_ch,32);
            vin_input = (vin_input-vin_zero)>>5;
            view_flag = 1;
            }
    meas_timing++;
    PIR1bits.TMR1IF=0;
    TMR1=40535;
    }
}

int main(int argc, char** argv) {
   
    unsigned int temp;
    long int voltage;
    long int current;
    float correction=0;
    unsigned char i,j;
    char cStr_Result[8];

    OSCCON = 0b01111010;    //16MHz internal RC

    TRISCbits.TRISC5=0;

    adc_init();
    lcd_init ();
    lcd_clear();
    tmr1_init();

while (1)
    {
    /*-----------------------------------------------------------
     *  +5V Vcc is selected as reference voltage for ADconversion,
     * next code will mathematically compansate reference voltage
     * using internal fixed voltage reference. Because code can by
     * started (view_flag) only once every 100ms after interrupt 
     * return, atomicity of global variables is not a problem at all.
     ----------------------------------------------------------*/

    if (view_flag)
        {
        view_flag=0;

        correction = 1024 / (float) fvr_input;
        vout_input = (unsigned int) ((float) vout_input * correction);
        iout_input = (unsigned int) ((float) iout_input * correction);
        vin_input  = (unsigned int) ((float) vin_input  * correction);

        /* Nasledujuci kod obchadza nutnost pouzitia printf pre zobrazenie */

        vout_input *= 3; // rozsah 0-30V zodpoveda hodnotam 0-3000
        iout_input *= 2;  // rozsah 0-2A zodpoveda hodnotam 0-2000

        lcd_setposition (1, 1);
        lcd_putchar('U');
        lcd_putchar('=');

        utoa(cStr_Result,vout_input,10);

        if (vout_input >=10000)              // >100V
            {
            lcd_putchar(cStr_Result[0]);
            lcd_putchar(cStr_Result[1]);
            lcd_putchar(cStr_Result[2]);
            lcd_putchar('.');
            lcd_putchar(cStr_Result[3]);
            }
        else if (vout_input >=1000)              // 10-30V   napr 10,1 V
            {
            lcd_putchar(cStr_Result[0]);
            lcd_putchar(cStr_Result[1]);
            lcd_putchar('.');
            lcd_putchar(cStr_Result[2]);
            lcd_putchar(cStr_Result[3]);
            }
        else if (vout_input >=100)          // 1-9V     napr 9.5 V
            {
            lcd_putchar(' ');
            lcd_putchar(cStr_Result[0]);
            lcd_putchar('.');
            lcd_putchar(cStr_Result[1]);
            lcd_putchar(cStr_Result[2]);
            }
        else if (vout_input >=10)           // 0-1V
            {
            lcd_putchar(' ');
            lcd_putchar('0');
            lcd_putchar('.');
            lcd_putchar(cStr_Result[0]);
            lcd_putchar(cStr_Result[1]);
            }
        
        lcd_putchar('V');

        lcd_setposition (2, 1);
        lcd_putchar('I');
        lcd_putchar('=');
        lcd_putchar(' ');
        utoa(cStr_Result,iout_input,10);
        if (iout_input>=1000)
            {
            lcd_putchar(cStr_Result[0]);
            lcd_putchar('.');
            lcd_putchar(cStr_Result[1]);
            lcd_putchar(cStr_Result[2]);
            }
        else if (iout_input>=100)
            {
            lcd_putchar('0');
            lcd_putchar('.');
            lcd_putchar(cStr_Result[0]);
            lcd_putchar(cStr_Result[1]);
            }
        else if (iout_input>=10)
            {
            lcd_putchar('0');
            lcd_putchar('.');
            lcd_putchar('0');
            lcd_putchar(cStr_Result[0]);
            }
        else
            {
            lcd_putchar('0');
            lcd_putchar('.');
            lcd_putchar('0');
            lcd_putchar('0');
            }

        lcd_putchar('A');
        }
    }
    return (EXIT_SUCCESS);
}